﻿// maindlg.cpp : implementation of the CMainDlg class
//
/////////////////////////////////////////////////////////////////////////////

#include "stdafx.h"

#include "aboutdlg.h"
#include "maindlg.h"
#include "StationSpyPlugin.h"

CMainDlg::CMainDlg(StationSpyPlugin *plugin, MQWindowBase& parent) : MQWindow(parent)
{
	setlocale(LC_ALL, ""); 

	m_pPlugin = plugin;
	/*
	// Set icons
	HICON hIcon = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXICON), ::GetSystemMetrics(SM_CYICON), LR_DEFAULTCOLOR);
	SetIcon(hIcon, TRUE);
	HICON hIconSmall = (HICON)::LoadImage(_Module.GetResourceInstance(), MAKEINTRESOURCE(IDR_MAINFRAME), 
		IMAGE_ICON, ::GetSystemMetrics(SM_CXSMICON), ::GetSystemMetrics(SM_CYSMICON), LR_DEFAULTCOLOR);
	SetIcon(hIconSmall, FALSE);
	*/

	SetTitle(L"Station Spy");
	SetOutSpace(0.4);

	MQFrame *mainFrame = CreateHorizontalFrame(this);
	mainFrame->SetVertLayout(MQWidgetBase::LAYOUT_FILL);

	m_MessageList = CreateListBox(mainFrame);
	m_MessageList->SetHorzLayout(MQWidgetBase::LAYOUT_FILL);
	m_MessageList->SetHintSizeRateX(20.0);

	MQFrame *sideFrame = CreateVerticalFrame(mainFrame);
	MQButton *okbtn = CreateButton(sideFrame, L"OK");
	MQButton *clearbtn = CreateButton(sideFrame, L"Clear");
	clearbtn->AddClickEvent(this, &CMainDlg::OnClear);
	
	m_CheckOnDraw = CreateCheckBox(sideFrame, L"OnDraw");
	m_CheckOnDraw->AddChangedEvent(this, &CMainDlg::OnCheckOnDraw);

	m_CheckOnUpdateScene = CreateCheckBox(sideFrame, L"OnUpdateScene");
	m_CheckOnUpdateScene->AddChangedEvent(this, &CMainDlg::OnCheckOnUpdateScene);

	m_CheckOnUpdateUndo = CreateCheckBox(sideFrame, L"OnUpdateUndo");
	m_CheckOnUpdateUndo->AddChangedEvent(this, &CMainDlg::OnCheckOnUpdateUndo);

	m_CheckWidgetEvent = CreateCheckBox(sideFrame, L"Widget events");
	m_CheckWidgetEvent->AddChangedEvent(this, &CMainDlg::OnCheckWidgetEvent);

	m_CheckMouseMove = CreateCheckBox(sideFrame, L"MouseMove");
	m_CheckMouseMove->AddChangedEvent(this, &CMainDlg::OnCheckMouseMove);

	MQButton *aboutbtn = CreateButton(sideFrame, L"About");
	aboutbtn->AddClickEvent(this, &CMainDlg::OnAppAbout);
	aboutbtn->SetFillBeforeRate(1);
}

BOOL CMainDlg::OnAppAbout(MQWidgetBase *sender, MQDocument doc)
{
	CAboutDlg dlg(*this);
	dlg.Execute();
	return FALSE;
}

BOOL CMainDlg::OnOK(MQWidgetBase *sender, MQDocument doc)
{
	SetVisible(false);
	return FALSE;
}

BOOL CMainDlg::OnClear(MQWidgetBase *sender, MQDocument doc)
{
	m_MessageList->ClearItems();
	return FALSE;
}

BOOL CMainDlg::OnCheckOnDraw(MQWidgetBase *sender, MQDocument doc)
{
	m_pPlugin->m_bOnDraw = m_CheckOnDraw->GetChecked();
	return FALSE;
}

BOOL CMainDlg::OnCheckOnUpdateScene(MQWidgetBase *sender, MQDocument doc)
{
	m_pPlugin->m_bOnUpdateScene = m_CheckOnUpdateScene->GetChecked();
	return FALSE;
}

BOOL CMainDlg::OnCheckOnUpdateUndo(MQWidgetBase *sender, MQDocument doc)
{
	m_pPlugin->m_bOnUpdateUndo = m_CheckOnUpdateUndo->GetChecked();
	return FALSE;
}

BOOL CMainDlg::OnCheckWidgetEvent(MQWidgetBase *sender, MQDocument doc)
{
	m_pPlugin->m_bWidgetEvents = m_CheckWidgetEvent->GetChecked();

	if(m_pPlugin->m_bWidgetEvents){
		m_MessageList->AddLeftDownEvent(this, &CMainDlg::OnListLeftDown);
		m_MessageList->AddLeftUpEvent(this, &CMainDlg::OnListLeftUp);
		m_MessageList->AddLeftDoubleClickEvent(this, &CMainDlg::OnListLeftDoubleClick);
		m_MessageList->AddMiddleDownEvent(this, &CMainDlg::OnListMiddleDown);
		m_MessageList->AddMiddleUpEvent(this, &CMainDlg::OnListMiddleUp);
		m_MessageList->AddMiddleDoubleClickEvent(this, &CMainDlg::OnListMiddleDoubleClick);
		m_MessageList->AddRightDownEvent(this, &CMainDlg::OnListRightDown);
		m_MessageList->AddRightUpEvent(this, &CMainDlg::OnListRightUp);
		m_MessageList->AddRightDoubleClickEvent(this, &CMainDlg::OnListRightDoubleClick);
		m_MessageList->AddMouseWheelEvent(this, &CMainDlg::OnListMouseWheel);
		m_MessageList->AddKeyDownEvent(this, &CMainDlg::OnListKeyDown);
		m_MessageList->AddKeyUpEvent(this, &CMainDlg::OnListKeyUp);
	}else{
		m_MessageList->RemoveLeftDownEvent(this, &CMainDlg::OnListLeftDown);
		m_MessageList->RemoveLeftUpEvent(this, &CMainDlg::OnListLeftUp);
		m_MessageList->RemoveLeftDoubleClickEvent(this, &CMainDlg::OnListLeftDoubleClick);
		m_MessageList->RemoveMiddleDownEvent(this, &CMainDlg::OnListMiddleDown);
		m_MessageList->RemoveMiddleUpEvent(this, &CMainDlg::OnListMiddleUp);
		m_MessageList->RemoveMiddleDoubleClickEvent(this, &CMainDlg::OnListMiddleDoubleClick);
		m_MessageList->RemoveRightDownEvent(this, &CMainDlg::OnListRightDown);
		m_MessageList->RemoveRightUpEvent(this, &CMainDlg::OnListRightUp);
		m_MessageList->RemoveRightDoubleClickEvent(this, &CMainDlg::OnListRightDoubleClick);
		m_MessageList->RemoveMouseWheelEvent(this, &CMainDlg::OnListMouseWheel);
		m_MessageList->RemoveKeyDownEvent(this, &CMainDlg::OnListKeyDown);
		m_MessageList->RemoveKeyUpEvent(this, &CMainDlg::OnListKeyUp);
	}

	return FALSE;
}

BOOL CMainDlg::OnCheckMouseMove(MQWidgetBase *sender, MQDocument doc)
{
	m_pPlugin->m_bMouseMove = m_CheckMouseMove->GetChecked();

	if(m_pPlugin->m_bMouseMove){
		m_MessageList->AddMouseMoveEvent(this, &CMainDlg::OnListMouseMove);
	}else{
		m_MessageList->RemoveMouseMoveEvent(this, &CMainDlg::OnListMouseMove);
	}

	return TRUE;
}

BOOL CMainDlg::OnListLeftDown(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"LeftDown : pos(%d %d)", param.ClientPos.x, param.ClientPos.y);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListLeftUp(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"LeftUp : pos(%d %d)", param.ClientPos.x, param.ClientPos.y);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListLeftDoubleClick(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"LeftDoubleClick : pos(%d %d)", param.ClientPos.x, param.ClientPos.y);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListMiddleDown(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"MiddleDown : pos(%d %d)", param.ClientPos.x, param.ClientPos.y);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListMiddleUp(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"MiddleUp : pos(%d %d)", param.ClientPos.x, param.ClientPos.y);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListMiddleDoubleClick(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"MiddleDoubleClick : pos(%d %d)", param.ClientPos.x, param.ClientPos.y);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListRightDown(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"RightDown : pos(%d %d)", param.ClientPos.x, param.ClientPos.y);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListRightUp(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"RightUp : pos(%d %d)", param.ClientPos.x, param.ClientPos.y);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListRightDoubleClick(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"RightDoubleClick : pos(%d %d)", param.ClientPos.x, param.ClientPos.y);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListMouseMove(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"MouseMove : pos(%d %d) %s%s%s", param.ClientPos.x, param.ClientPos.y, 
		param.LButton?L"L":L"", param.MButton?L"M":L"", param.RButton?L"R":L"");
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListMouseWheel(MQWidgetBase *sender, MQDocument doc, MQWidgetMouseParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"MouseWheel : %d", param.Wheel);
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListKeyDown(MQWidgetBase *sender, MQDocument doc, MQWidgetKeyParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"KeyDown : %d %s", param.Key, param.AutoRepeat?L"repeat":L"");
	AddMessage(buf);
	return FALSE;
}

BOOL CMainDlg::OnListKeyUp(MQWidgetBase *sender, MQDocument doc, MQWidgetKeyParam& param)
{
	wchar_t buf[256];
	swprintf_s(buf, L"KeyUp : %d", param.Key);
	AddMessage(buf);
	return FALSE;
}

// Add a message log
// メッセージログの追加
void CMainDlg::AddMessage(const wchar_t *message)
{
	int index = m_MessageList->AddItem(message);
	m_MessageList->MakeItemVisible(index);
}

void CMainDlg::CloseDialog(int nVal)
{
	// Hide the window
	// ウインドウを隠す
	SetVisible(false);

	// Notify from the plug-in
	// プラグインから通知
	m_pPlugin->m_bActivate = false;
	m_pPlugin->WindowClose();
}
