﻿//---------------------------------------------------------------------------
//
//   TTFont
//
//          Copyright(C) 1999-2013, tetraface Inc.
//
//     Sample of Create plug-in for creating a string with True-Type font.
//     This is just a sample. The precision of the triangulation is not 
//    enough.
//     Created DLL must be installed in "Plugins\Create" directory.
//
//    　True-Type Fontから文字列を作成するクリエイトプラグインのサンプル。
//    三角形分割処理はそれほど精度が高くないのでご注意ください。
//    　作成したDLLは"Plugins\Create"フォルダに入れる必要があります。
//
//
//---------------------------------------------------------------------------

#define WIN32_LEAN_AND_MEAN
#include <windows.h>
#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <set>
#include "MQPlugin.h"
#include "MQBasePlugin.h"
#include "MQSetting.h"
#include "MQ3DLib.h"
#include "MQWidget.h"
#include "Triangulate.h"
//#include "resource.h"


HINSTANCE hInstance;

std::wstring Utf8ToWide(const char *ptr)
{
	if(ptr == NULL) return std::wstring();

	int lenw = MultiByteToWideChar(CP_UTF8,0,ptr,-1,NULL,0);
	wchar_t *strw = (wchar_t*)malloc(sizeof(wchar_t) * (lenw+1));
	MultiByteToWideChar(CP_UTF8,0,ptr,-1,strw,lenw);

	std::wstring str(strw);
	free(strw);
	return str;
}

std::string WideToUtf8(const wchar_t *ptr)
{
	if(ptr == NULL) return std::string();

	int lenu = WideCharToMultiByte(CP_UTF8,0,ptr,-1,NULL,0,NULL,NULL);
	char *stru = (char*)malloc(lenu+1);
	WideCharToMultiByte(CP_UTF8,0,ptr,-1,stru,lenu,NULL,NULL);

	std::string str(stru);
	free(stru);
	return str;
}

std::string WideToAnsi(const wchar_t *ptr)
{
	if(ptr == NULL) return std::string();

	int lenu = WideCharToMultiByte(CP_ACP,0,ptr,-1,NULL,0,NULL,NULL);
	char *stru = (char*)malloc(lenu+1);
	WideCharToMultiByte(CP_ACP,0,ptr,-1,stru,lenu,NULL,NULL);

	std::string str(stru);
	free(stru);
	return str;
}


//---------------------------------------------------------------------------
//  DllMain
//---------------------------------------------------------------------------
BOOL APIENTRY DllMain( HANDLE hModule, 
                       DWORD  ul_reason_for_call, 
                       LPVOID lpReserved
					 )
{
	// リソース表示に必要なので、インスタンスを保存しておく
	// Store the instance because it is necessary for displaying resources.
	hInstance = (HINSTANCE)hModule;

	// プラグインとしては特に必要な処理はないので、何もせずにTRUEを返す
	// There is nothing to do for a plugin. So just return TRUE.
    return TRUE;
}


//---------------------------------------------------------------------------
//  class TTFontPlugin
//---------------------------------------------------------------------------
class TTFontPlugin : public MQCreatePlugin
{
public:
	TTFontPlugin();
	~TTFontPlugin();

	void GetPlugInID(DWORD *Product, DWORD *ID) override;
	const char *GetPlugInName(void) override;
	const char *EnumString(int index) override;
	BOOL Execute(int index, MQDocument doc) override;

private:
	BOOL CreateFont(MQDocument doc);
};


MQBasePlugin *GetPluginClass()
{
	static TTFontPlugin plugin;
	return &plugin;
}


TTFontPlugin::TTFontPlugin()
{
}

TTFontPlugin::~TTFontPlugin()
{
}

//---------------------------------------------------------------------------
//  GetPlugInID
//    プラグインIDを返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
void TTFontPlugin::GetPlugInID(DWORD *Product, DWORD *ID)
{
	// プロダクト名(制作者名)とIDを、全部で64bitの値として返す
	// 値は他と重複しないようなランダムなもので良い
	*Product = 0x56A31D20;
	*ID      = 0x5EAC34E0;
}

//---------------------------------------------------------------------------
//  GetPlugInName
//    プラグイン名を返す。
//    この関数は[プラグインについて]表示時に呼び出される。
//---------------------------------------------------------------------------
const char *TTFontPlugin::GetPlugInName(void)
{
	// プラグイン名
	return "Create Text with TrueType Font      Copyright(C) 1999-2013, tetraface Inc.";
}

//---------------------------------------------------------------------------
//  EnumString
//    ポップアップメニューに表示される文字列を返す。
//    この関数は起動時に呼び出される。
//---------------------------------------------------------------------------
const char *TTFontPlugin::EnumString(int index)
{
	switch(index){
	case 0:
		// Note: following $res$ means the string uses system resource string.
		return "$res$TTFont.Title";
	}
	return NULL;
}

//---------------------------------------------------------------------------
//  Execute
//    メニューから選択されたときに呼び出される。
//---------------------------------------------------------------------------
BOOL TTFontPlugin::Execute(int index, MQDocument doc)
{
	switch(index){
	case 0: return CreateFont(doc);
	}
	return FALSE;
}


typedef enum SPTYPE {SPEdge, SPSpline} SPTYPE;
typedef struct _SPLINEPOINT {
	POINTFX p;
	SPTYPE type;
} SPLINEPOINT;

TTPoint PointFxToPoint(POINTFX f)
{
	TTPoint p;
	p.x = (float)f.x.fract / 65536.f + (float)f.x.value;
	p.y = (float)f.y.fract / 65536.f + (float)f.y.value;
	return p;
};


class MOutline
{
public:
	TTPolygon **Polygons;
	int Count;

	MOutline() {
		Polygons = NULL;
		Count = 0;
	}
	virtual ~MOutline(){
		Clear();
	}

	void Clear(void){
		if(Polygons) {
			for(int i=0; i<Count; i++)
				delete Polygons[i];
			free(Polygons);
			Polygons = NULL;
			Count = 0;
		}
	}

	int Add(TTPolygon *NewPoly)
	{
		Polygons = (TTPolygon **)realloc(Polygons, sizeof(TTPolygon *) * (Count+1));
		Polygons[Count++] = NewPoly;
		return Count; //return count of faces
	}
	int Delete(int index)
	{
		if(index < Count){
			delete Polygons[index];
			for(int i=index; i<Count-1; i++){
				Polygons[i] = Polygons[i+1];
			}
			Count--;
		}
		return Count;
	}

	bool GetOutline(HDC hdc, UINT uChar, int segment);
	int CheckCrossingLine(TTPoint l1, TTPoint l2);
	int CheckCrossingLine(TTPolygon *poly1, int index1, TTPolygon *poly2, int index2);
	bool Analyze(MQObject obj, float offset_x);
	bool IsInside(int out_index, int in_index);
};

bool MOutline::GetOutline(HDC hdc, UINT uChar, int segment)
{
	int ObjCount=0;
	GLYPHMETRICS gm;
	TEXTMETRIC tm;
	MAT2 mat2 = {{0,1}, {0,0}, {0,0}, {0,1}};

	//get size of memory for outline
	GetGlyphOutline(hdc, uChar, GGO_METRICS, &gm, 0, NULL, &mat2);
	DWORD BufSize = GetGlyphOutline(hdc, uChar, GGO_NATIVE, &gm, 0, NULL, &mat2);
	if(BufSize == 0 || BufSize == GDI_ERROR)
		return false;

	//alloc buffer
	void *Buffer = malloc(BufSize);
	if(Buffer == NULL)
		return false;

	//get outline
	if(GetGlyphOutline(hdc, uChar, GGO_NATIVE, &gm, BufSize, Buffer, &mat2) == GDI_ERROR)
	{
		free(Buffer);
		return false;
	}

	//get metrics
	float descent = 0;
	if(GetTextMetrics(hdc, &tm))
		descent = (float)tm.tmDescent / tm.tmHeight;

	int LastSize = BufSize;
	void *Ptr = Buffer;
	int PtfxCount;

	SPLINEPOINT *Ptfx = (SPLINEPOINT *)malloc(sizeof(SPLINEPOINT) * 1024);

	//analyze outline
	for(; LastSize>0; ObjCount++)
	{
		TTPOLYGONHEADER *Ttph = (TTPOLYGONHEADER *)Ptr;
		LastSize -= Ttph->cb;
		Ptr = ((char*)Ptr + Ttph->cb);

		if(Ttph->dwType != TT_POLYGON_TYPE)
			continue;
		int LastPoly = Ttph->cb - sizeof(TTPOLYGONHEADER);
		Ptfx[0].p = Ttph->pfxStart;
		Ptfx[0].type = SPEdge;
		PtfxCount = 1;

		//create poly-face
		TTPolygon *Poly = new TTPolygon;

		//get index
		TTPOLYCURVE *Ttpc = (TTPOLYCURVE *)&Ttph[1];
		for(;LastPoly>0;)
		{
			int pc = Ttpc->cpfx;
			LastPoly -= (pc-1)*sizeof(POINTFX)+sizeof(TTPOLYCURVE);
			POINTFX *pfx = Ttpc->apfx;
			switch(Ttpc->wType){
			case TT_PRIM_LINE:{
				for(int i=0; i<pc; i++,pfx++){
					Ptfx[i+PtfxCount].p = *pfx;
					Ptfx[i+PtfxCount].type = SPEdge;
				}
				PtfxCount += pc;
				break;}
			case TT_PRIM_QSPLINE:{
				for(int i=0; i<pc; i++,pfx++){
					Ptfx[PtfxCount+i].p = *pfx;
					Ptfx[PtfxCount+i].type = SPSpline;
				}
				PtfxCount += pc;
				Ptfx[PtfxCount-1].type = SPEdge;
				break;}
			}
			Ttpc = (TTPOLYCURVE *)pfx;
		}
		if(memcmp(&Ptfx[0].p, &Ptfx[PtfxCount-1].p, sizeof(POINTFX)) == 0)
			PtfxCount--;
		Ptfx[PtfxCount].p = Ptfx[0].p;
		Ptfx[PtfxCount].type = SPEdge;

		//set points
		for(int i=0; i<PtfxCount; i++)
		{
			TTPoint pp, pa, pb, pc;

			switch(Ptfx[i].type){
			case SPEdge:
				pp = PointFxToPoint(Ptfx[i].p);
				pp.x = pp.x / 2048.f;
				pp.y = (pp.y + descent) / 2048.f;
				Poly->Add(pp);
				break;
			case SPSpline:
				pb = PointFxToPoint(Ptfx[i].p);
				pa = PointFxToPoint(Ptfx[i-1].p);
				if(Ptfx[i-1].type != SPEdge) 
					pa = (pa + pb) / 2;
				pc = PointFxToPoint(Ptfx[i+1].p);
				if(Ptfx[i+1].type != SPEdge) 
					pc = (pc + pb) / 2;

				for(int s=1; s<segment; s++)
				{
					float t = (float)s / (float)segment;
					pp = (pa-2*pb+pc)*t*t + 2*(pb-pa)*t + pa;
					pp.x = pp.x / 2048.f;
					pp.y = (pp.y + descent) / 2048.f;
					Poly->Add(pp);
				}
				break;
			}
		}

		Add(Poly);
	}

	free(Ptfx);
	free(Buffer);

	return true;
}

int MOutline::CheckCrossingLine(TTPoint l1, TTPoint l2)
{
	for(int i=0; i<Count; i++)
	{
		TTPolygon *poly = Polygons[i];
		for(int j=0; j<poly->ptscount; j++)
		{
			TTPoint p1 = poly->pts[j];
			TTPoint p2 = poly->pts[j+1<poly->ptscount ? j+1 : 0];
			float n1 = (p1.x-l1.x) * (p2.x-l1.y) - (p1.y-l1.y) * (p2.x-l1.x);
			float n2 = (p1.x-l2.x) * (p2.x-l2.y) - (p1.y-l2.y) * (p2.x-l2.x);
			if(n1 * n2 < 0)
				return i;
		}
	}

	return -1;
}

int MOutline::CheckCrossingLine(TTPolygon *poly1, int index1, TTPolygon *poly2, int index2)
{
	TTPoint l1 = poly1->pts[poly1->indices->index[index1]];
	TTPoint l2 = poly2->pts[poly2->indices->index[index2]];
	for(int i=0; i<Count; i++)
	{
		TTPolygon *poly = Polygons[i];
		int icount = poly->indices->count;
		int *index = poly->indices->index;
		for(int j=0; j<icount; j++)
		{
			TTPoint p1 = poly->pts[index[j]];
			TTPoint p2 = poly->pts[index[j+1<icount ? j+1 : 0]];
			float n1 = (p1.x-l1.x) * (p2.y-l1.y) - (p1.y-l1.y) * (p2.x-l1.x);
			float n2 = (p1.x-l2.x) * (p2.y-l2.y) - (p1.y-l2.y) * (p2.x-l2.x);
			float n3 = (p1.x-l1.x) * (p1.y-l2.y) - (p1.y-l1.y) * (p1.x-l2.x);
			float n4 = (p2.x-l1.x) * (p2.y-l2.y) - (p2.y-l1.y) * (p2.x-l2.x);
			if(n1 * n2 <= 0 && n3 * n4<=0) {
				if(poly == poly1 && (j == index1 || (j+1)%icount == index1))
					continue;
				if(poly == poly2 && (j == index2 || (j+1)%icount == index2))
					continue;
				return i;
			}
		}
	}

	return -1;
}

bool MOutline::IsInside(int out_index, int in_index)
{
	if(out_index == in_index)
		return false;

	TTPolygon *outside, *inside;
	int outcount;
	TTPoint *op, ip;
	int i;

	outside = Polygons[out_index];
	outcount = outside->indices->count;

	// copy points to buffer
	op = (TTPoint *)malloc(sizeof(TTPoint) * (outcount+2));
	for(i=0; i<outcount; i++)
		op[i] = outside->pts[outside->indices->index[i]];
	op[outside->indices->count]   = op[0];
	op[outside->indices->count+1] = op[1];

	inside = Polygons[in_index];
	ip = inside->pts[0]; // check only one point

	int flag=0; //cross count  inside(1) or outside(0)
	int endv = outcount;
	char *lchk = (char *)malloc(sizeof(char) * outcount);
	memset(lchk, 0, sizeof(char) * outcount);

	for(i=0; i<outcount; i++)
	{
		float v = (ip.y-op[i].y) * (ip.y-op[i+1].y);
		if(v < 0) {
			float t = (ip.y - op[i].y) / (op[i+1].y - op[i].y);
			if(ip.x < op[i].x*(1-t) + op[i+1].x*t) {
				lchk[i] = 1;
				flag++;
			} else {
				lchk[i] = -1;
			}
		} else if(v > 0)
			lchk[i] = -1;
	}

	for(i=0; i<outcount; i++)
	{
		if(lchk[i] != 0)
			continue;

		if(ip.y != op[i].y)
			continue;

		int v1,v2;
		for(v1=i; v1!=v1-outcount; v1--) {
			if(ip.y != op[(v1+outcount)%outcount].y)
				break;
		}
		for(v2=i; v2!=v2+outcount; v2++) {
			if(ip.y != op[(v2+outcount)%outcount].y)
				break;
		}
		float v = (ip.y-op[(v1+outcount)%outcount].y) * (ip.y-op[v2%outcount].y);
		if(v < 0 && ip.x<op[(v1+1)%outcount].x) {
			for(; v1!=i; v1++)
				lchk[(v1+outcount)%outcount] = 1;
			for(--v2; v2!=i; v2--)
				lchk[v2%outcount] = 1;
			lchk[i] = 1;
			flag++;
		} else {
			for(; v1!=i; v1++)
				lchk[(v1+outcount)%outcount] = -1;
			for(--v2; v2!=i; v2--)
				lchk[v2%outcount] = -1;
			lchk[i] = -1;
		}
	}

	free(lchk);

	return (flag & 1);
}

bool MOutline::Analyze(MQObject obj, float offset_x)
{
	int i,j,k,l,m;

	// align clockwise
	for(i=0; i<Count; i++)
	{
		TTPolygon *poly = Polygons[i];
		if(poly->ptscount >= 3) {
			if(!poly->IsClockwise())
				poly->Invert();
		}
	}

	// initialize indices of points
	for(i=0; i<Count; i++)
		Polygons[i]->InitIndex();

	// piled outlines must be separated, but not implement

	// analyze inside
START_ANALYZE:
	for(i=0; i<Count; i++)
	{
#if 1
		// if this polygon is in other polygon, ignore
		for(j=0; j<Count; j++)
			if(IsInside(j, i))
				break;
		if(j < Count)
			continue;
#endif

		// check if other polygon is inside
		for(j=0; j<Count; j++)
		{
			if(!IsInside(i, j))
				continue;

#if 1
			TTPolygon *outside = Polygons[i];
			TTPolygon *inside = Polygons[j];
			int opc = outside->indices->count;
			int ipc = inside->indices->count;
			// search uncrossing line
			for(k=0; k<opc; k++)
			{
				for(l=0; l<ipc; l++)
				{
					if(CheckCrossingLine(outside, k, inside, l) < 0)
					{
						// merge points
						int *nvi = (int *)malloc(sizeof(int) * inside->ptscount);
						for(m=0; m<inside->ptscount; m++)
							nvi[m] = outside->Add(inside->pts[m]);

						// merge outlines
						int n=1;
						for(m=l; m>=0; m--,n++)
							outside->indices->Insert(k+n, nvi[inside->indices->index[m]]);
						for(m=ipc-1; m>=l; m--,n++)
							outside->indices->Insert(k+n, nvi[inside->indices->index[m]]);
						outside->indices->Insert(k+n, outside->indices->index[k]);

						free(nvi);
						Delete(j);
						goto START_ANALYZE;
					}
				}
			}
#else
			Delete(j);
			goto START_ANALYZE;
#endif
		}
	}

	// register polygons to object
	for(i=0; i<Count; i++)
	{
		TTPolygon *poly = Polygons[i];
		if(poly->ptscount < 2)
			continue;

		// add vertices
		int *vi = (int *)malloc(sizeof(int) * poly->ptscount);
		for(j=0; j<poly->ptscount; j++)
			vi[j] = obj->AddVertex( MQPoint(poly->pts[j].x + offset_x, poly->pts[j].y, 0.f) );

		// triangulate
		poly->Triangulate();
		for(int i=0; i<poly->triangle->count; i++)
		{
			Indices *indices = poly->triangle->index[i];
			int fvi[3];
			fvi[0] = vi[indices->index[0]];
			fvi[1] = vi[indices->index[2]];
			fvi[2] = vi[indices->index[1]];
			obj->AddFace(3, fvi);
		}

		free(vi);
	}

	return true;
}

bool CreateFont(MQObject obj, HDC hdc, const wchar_t *wstr, int segment)
{
	int len = (int)wcslen(wstr);
	if(len == 0)
		return false;

	// set object from font
	float offset_x = 0;
	for(int i=0; i<len; i++)
	{
		//convert unicode character to multi-byte
		char mchar[4];
		memset(mchar, 0, sizeof(mchar));
		WideCharToMultiByte(CP_OEMCP, 0, &wstr[i], 
			1, mchar, sizeof(mchar), NULL, NULL);

		//convert multi-byte to glyph
		UINT gchar=0;
		for(int i=0; i<4 && mchar[i]!=0; i++){
			gchar<<=8;
			unsigned char c = mchar[i];
			gchar |= (UINT)c;
		}

		// get outline
		MOutline outline;
		if(!outline.GetOutline(hdc, gchar, segment))
			continue;

		outline.Analyze(obj, offset_x);

		// get width and update offset
		ABC Abc;
		GetCharABCWidths(hdc, gchar, gchar, &Abc);
		offset_x += (float)(Abc.abcA + Abc.abcB + Abc.abcC) / 2048.f;
	}

	//move to client area


	return true;
}

MQObject CreateTextObject(const wchar_t *text, const wchar_t *font, int segment)
{
	MQObject obj = MQ_CreateObject();

	// create DC for drawing font
	HDC hdc = CreateDCW(L"DISPLAY",NULL,NULL,NULL);

	// set info of font
	LOGFONTW lf;
	memset(&lf, 0, sizeof(LOGFONTW));
	lf.lfHeight = 2048;
	lf.lfWidth  = 0;
	lf.lfEscapement = 0;
	lf.lfWeight = FW_NORMAL;
	lf.lfItalic = 0;
	lf.lfUnderline = 0;
	lf.lfStrikeOut = 0;
	lf.lfCharSet   = DEFAULT_CHARSET;
	lf.lfOutPrecision = OUT_DEFAULT_PRECIS; //OUT_STROKE_PRECIS;
	lf.lfClipPrecision = CLIP_DEFAULT_PRECIS;
	lf.lfQuality = DEFAULT_QUALITY; //PROOF_QUALITY;
	lf.lfPitchAndFamily = FF_DONTCARE | DEFAULT_PITCH;
	wcsncpy_s(lf.lfFaceName, font, LF_FACESIZE-1);

	// create font
	HFONT hfont = CreateFontIndirectW(&lf);
	HFONT hfontold = (HFONT)SelectObject(hdc, hfont);

	CreateFont(obj, hdc, text, segment);

	// delete objects
	SelectObject(hdc, hfontold);
	DeleteObject(hfont);
	DeleteDC(hdc);

	return obj;
}

int CALLBACK FontEnumProc(LPLOGFONTW lpLogFont, LPTEXTMETRIC lpTextMetric, DWORD dwType, LPARAM lpData)
{
	if(dwType == TRUETYPE_FONTTYPE)
	{
		if(lpLogFont->lfFaceName[0] != L'@'){
			std::set<std::wstring> *fonts = (std::set<std::wstring>*)lpData;
			fonts->insert(std::wstring(lpLogFont->lfFaceName));
		}
	}

	return 1; // continue to enumeration
}

class TTFontDialog : public MQDialog
{
public:
	TTFontDialog(MQWindowBase& parent, TTFontPlugin *plugin);
	~TTFontDialog();

	BOOL ThickCheckChanged(MQWidgetBase *sender, MQDocument doc);

	MQComboBox *fonts;
	MQEdit *edit;
	MQSpinBox *segment;
	MQDoubleSpinBox *size;
	MQCheckBox *thick;
	MQDoubleSpinBox *thickness;
};

TTFontDialog::TTFontDialog(MQWindowBase& parent, TTFontPlugin *plugin) : MQDialog(parent)
{
	SetTitle(plugin->GetResourceString("TTFont.Title"));

	MQFrame *mainFrame = CreateHorizontalFrame(this);

	MQFrame *paramFrame = CreateVerticalFrame(mainFrame);
	paramFrame->SetHorzLayout(MQWidgetBase::LAYOUT_FILL);
	
	// Font
	MQFrame *fontFrame = CreateVerticalFrame(paramFrame);
	fontFrame->SetInSpace(0);
	MQLabel *label = CreateLabel(fontFrame, plugin->GetResourceString("TTFont.FontName"));
	fonts = CreateComboBox(fontFrame);
	fonts->SetHintSizeRateX(12);
	fonts->SetHorzLayout(MQWidgetBase::LAYOUT_FILL);

	LOGFONTW lf;
	if(!::SystemParametersInfoW(SPI_GETICONTITLELOGFONT, sizeof(LOGFONT), &lf, 0)){
		lf.lfFaceName[0] = L'\0';
	}

	std::set<std::wstring> font_array;
	HDC hdc = GetDC(NULL);
	EnumFontsW(hdc, NULL, (FONTENUMPROCW)FontEnumProc, (LPARAM)&font_array);
	ReleaseDC(NULL, hdc);

	for(std::set<std::wstring>::iterator it = font_array.begin(); it != font_array.end(); ++it){
		int index = fonts->AddItem(*it);
		if(wcscmp((*it).c_str(), lf.lfFaceName) == 0){
			fonts->SetCurrentIndex(index);
		}
	}

	// Text
	MQFrame *editFrame = CreateVerticalFrame(paramFrame);
	editFrame->SetInSpace(0);
	label = CreateLabel(editFrame, plugin->GetResourceString("TTFont.FontString"));
	edit = CreateEdit(editFrame);
	edit->SetHorzLayout(MQWidgetBase::LAYOUT_FILL);

	MQFrame *frame = CreateHorizontalFrame(paramFrame);
	frame->SetMatrixColumn(2);

	// Segment
	label = CreateLabel(frame, plugin->GetResourceString("TTFont.Segment"));
	segment = CreateSpinBox(frame);
	segment->SetMin(1);
	segment->SetMax(20);
	segment->SetPosition(5);

	// Size
	label = CreateLabel(frame, plugin->GetResourceString("TTFont.Size"));
	size = CreateDoubleSpinBox(frame);
	size->SetMin(1e-8);
	size->SetMax(1e+8);
	size->SetPosition(100);

	// Thickness
	thick = CreateCheckBox(frame, plugin->GetResourceString("TTFont.Thickness"));
	thick->AddChangedEvent(this, &TTFontDialog::ThickCheckChanged);
	thickness = CreateDoubleSpinBox(frame);
	thickness->SetMin(1e-8);
	thickness->SetMax(1e+8);
	thickness->SetPosition(5);
	thickness->SetEnabled(thick->GetChecked());

	MQFrame *sideFrame = CreateVerticalFrame(mainFrame);

	// OK Button
	MQButton *okbtn = CreateButton(sideFrame, plugin->GetResourceString("OK"));
	okbtn->SetDefault(true);
	okbtn->SetModalResult(MQDialog::DIALOG_OK);

	MQButton *cancelbtn = CreateButton(sideFrame, plugin->GetResourceString("Cancel"));
	cancelbtn->SetCancel(true);
	cancelbtn->SetModalResult(MQDialog::DIALOG_CANCEL);
}

TTFontDialog::~TTFontDialog()
{
}

BOOL TTFontDialog::ThickCheckChanged(MQWidgetBase *sender, MQDocument doc)
{
	thickness->SetEnabled(thick->GetChecked());
	return FALSE;
}

BOOL TTFontPlugin::CreateFont(MQDocument doc)
{
	// Load the settings.
	std::wstring font;
	int segment = 5;
	float size = 100.0f;
	bool thick = true;
	float thickness = 10.0f;
	MQSetting *setting = OpenSetting();
	if(setting != NULL){
		setting->Load("Font", font, font);
		setting->Load("Segment", segment, segment);
		setting->Load("Size", size, size);
		setting->Load("Thick", thick, thick);
		setting->Load("Thickness", thickness, thickness);
		CloseSetting(setting);
	}

	// ダイアログを表示
	// Show a dialog.
	MQWindow mainwin = MQWindow::GetMainWindow();
	TTFontDialog dialog(mainwin, this);
	if(font.length() > 0){
		for(int i=0; i<dialog.fonts->GetItemCount(); i++){
			if(dialog.fonts->GetItem(i) == font){
				dialog.fonts->SetCurrentIndex(i);
				break;
			}
		}
	}
	dialog.segment->SetPosition(segment);
	dialog.size->SetPosition(size);
	dialog.thick->SetChecked(thick);
	dialog.thickness->SetEnabled(thick);
	dialog.thickness->SetPosition(thickness);
	if(dialog.Execute() != MQDialog::DIALOG_OK){
		return FALSE;
	}

	// Get parameters from the dialog.
	std::wstring text = dialog.edit->GetText();
	font = dialog.fonts->GetItem(dialog.fonts->GetCurrentIndex());
	segment = dialog.segment->GetPosition();
	size = (float)dialog.size->GetPosition();
	thick = dialog.thick->GetChecked();
	thickness = (float)dialog.thickness->GetPosition();
	
	// Save the settings.
	setting = OpenSetting();
	if(setting != NULL){
		setting->Save("Font", font);
		setting->Save("Segment", segment);
		setting->Save("Size", size);
		setting->Save("Thick", thick);
		setting->Save("Thickness", thickness);
		CloseSetting(setting);
	}

	// 文字列オブジェクトを作成
	// Create an object for a text.
	MQObject obj = CreateTextObject(text.c_str(), font.c_str(), segment);
	if(obj == NULL)
		return FALSE;

	// Scaling
	int vertex_count = obj->GetVertexCount();
	for(int i=0; i<vertex_count; i++){
		MQPoint p = obj->GetVertex(i) * size;
		if(thick) p.z += thickness * 0.5f;
		obj->SetVertex(i, p);
	}

	if(thick){
		int face_count = obj->GetFaceCount();
		if(thickness > 0){
			MQObjEdge pair(obj);
			// Create back side faces.
			for(int i=0; i<vertex_count; i++){
				MQPoint p = obj->GetVertex(i) - MQPoint(0,0,thickness);
				obj->AddVertex(p);
			}
			for(int i=0; i<face_count; i++){
				int vi[3];
				obj->GetFacePointArray(i, vi);
				std::swap(vi[0], vi[2]);
				for(int j=0; j<3; j++){
					vi[j] += vertex_count;
				}
				obj->AddFace(3, vi);
			}
			// Create side faces for the thickness.
			for(int i=0; i<face_count; i++){
				for(int j=0; j<3; j++){
					int pf, pl;
					if(!pair.getPair(i, j, pf, pl)){
						int vi[3];
						obj->GetFacePointArray(i, vi);
						int nvi[4];
						nvi[0] = vi[j];
						nvi[1] = vi[j]+vertex_count;
						nvi[2] = vi[(j+1)%3]+vertex_count;
						nvi[3] = vi[(j+1)%3];
						obj->AddFace(4, nvi);
					}
				}
			}
		}else{
			// Create back side faces.
			for(int i=0; i<face_count; i++){
				int vi[3];
				obj->GetFacePointArray(i, vi);
				std::swap(vi[0], vi[2]);
				obj->AddFace(3, vi);
			}
		}
	}

	// ドキュメントにオブジェクトを登録し、カレントにする
	// Add the object, and set a current.
	int object_index = doc->AddObject(obj);
	doc->SetCurrentObjectIndex(object_index);

	return TRUE;
}

