﻿
#include "stdafx.h"
#include "mainwin.h"
//#include "WidgetTestPlugin.h"


MainWindow::MainWindow(WidgetTestPlugin *plugin, MQWindowBase& parent) : MQWindow(parent)
{
	setlocale(LC_ALL, ""); 

	m_pPlugin = plugin;

	SetTitle(L"Widget Test");
	SetOutSpace(0.4);

	m_Tab = CreateTab(this);
	m_Tab->AddChangedEvent(this, &MainWindow::TabChanged);

	m_Frame[0] = CreateButtonFrame(m_Tab);
	m_Tab->SetTabTitle(0, L"Button");

	m_Frame[1] = CreateCheckFrame(m_Tab);
	m_Tab->SetTabTitle(1, L"Check");

	m_Frame[2] = CreateComboFrame(m_Tab);
	m_Tab->SetTabTitle(2, L"Combo");

	m_Frame[3] = CreateListFrame(m_Tab);
	m_Tab->SetTabTitle(3, L"List");

	m_Frame[4] = CreateLabelFrame(m_Tab);
	m_Tab->SetTabTitle(4, L"Label");

	m_Frame[5] = CreateEditFrame(m_Tab);
	m_Tab->SetTabTitle(5, L"Edit");

	m_Frame[6] = CreateSpinBoxFrame(m_Tab);
	m_Tab->SetTabTitle(6, L"SpinBox");

	m_Frame[7] = CreateSliderFrame(m_Tab);
	m_Tab->SetTabTitle(7, L"Slider/ScrollBar");

	m_Frame[8] = CreateColorFrame(m_Tab);
	m_Tab->SetTabTitle(8, L"Color");

	m_BottomLabel = CreateLabel(this);
	m_BottomLabel->SetHintSizeRateY(4);
	m_BottomLabel->SetFrame(true);

}


MQFrame *MainWindow::CreateButtonFrame(MQWidgetBase *parent)
{
	MQFrame *root = CreateVerticalFrame(parent);

	MQFrame *mtx_frame = CreateHorizontalFrame(root);
	mtx_frame->SetMatrixColumn(3);
	mtx_frame->SetInSpace(0);
	for(int i=0; i<9; i++){
		wchar_t name[16];
		swprintf_s(name, L"Button %d", i+1);

		m_Buttons[i] = CreateButton(mtx_frame);
		m_Buttons[i]->SetText(name);
		//m_Buttons[i]->SetGroup(true);
		m_Buttons[i]->AddClickEvent(this, &MainWindow::ButtonClick);
	}

	m_ToggleButton = CreateButton(root, L"Toggle");
	m_ToggleButton->SetToggle(true);
	m_ToggleButton->AddClickEvent(this, &MainWindow::ToggleButtonClick);

	m_RepeatButton = CreateButton(root, L"Repeat");
	m_RepeatButton->SetRepeat(true);
	m_RepeatButton->AddClickEvent(this, &MainWindow::RepeatButtonClick);
	m_RepeatButton->AddRepeatEvent(this, &MainWindow::RepeatButtonRepeat);

	MQButton *btn = CreateButton(root, L"OpenFileDialog");
	btn->AddClickEvent(this, &MainWindow::OpenFileDialog);

	return root;
}

MQFrame *MainWindow::CreateCheckFrame(MQWidgetBase *parent)
{
	MQFrame *root = CreateVerticalFrame(parent);

	m_Check = CreateCheckBox(root);
	m_Check->SetText(L"Check1");
	m_Check->AddChangedEvent(this, &MainWindow::CheckChanged);

	for(int i=0; i<3; i++){
		wchar_t name[16];
		swprintf_s(name, L"Radio %d", i+1);

		m_Radio[i] = CreateRadioButton(root);
		m_Radio[i]->SetText(name);
		m_Radio[i]->AddChangedEvent(this, &MainWindow::RadioChanged);
	}
	m_Radio[0]->SetChecked(true);

	return root;
}

MQFrame *MainWindow::CreateComboFrame(MQWidgetBase *parent)
{
	MQFrame *root = CreateVerticalFrame(parent);

	m_Combo = CreateComboBox(root);
	m_Combo->AddItem(L"Default1", 1);
	m_Combo->AddItem(L"Default2", 2);
	m_Combo->AddItem(L"Default3", 3);
	m_Combo->AddChangedEvent(this, &MainWindow::ComboChanged);

	MQFrame *frame = CreateHorizontalFrame(root);
	frame->SetUniformSize(true);
	m_ComboAdd = CreateButton(frame, L"Add");
	m_ComboAdd->AddClickEvent(this, &MainWindow::ComboAddClick);
	m_ComboDelete = CreateButton(frame, L"Delete");
	m_ComboDelete->AddClickEvent(this, &MainWindow::ComboDeleteClick);
	m_ComboClear = CreateButton(frame, L"Clear");
	m_ComboClear->AddClickEvent(this, &MainWindow::ComboClearClick);

	return root;
}

MQFrame *MainWindow::CreateListFrame(MQWidgetBase *parent)
{
	MQFrame *root = CreateVerticalFrame(parent);

	m_List = CreateListBox(root);
	m_List->AddItem(L"Default1", 1);
	m_List->AddItem(L"Default2", 2);
	m_List->AddItem(L"Default3", 3);
	m_List->AddChangedEvent(this, &MainWindow::ListChanged);

	MQFrame *frame = CreateHorizontalFrame(root);
	frame->SetUniformSize(true);
	m_ListAdd = CreateButton(frame, L"Add");
	m_ListAdd->AddClickEvent(this, &MainWindow::ListAddClick);
	m_ListDelete = CreateButton(frame, L"Delete");
	m_ListDelete->AddClickEvent(this, &MainWindow::ListDeleteClick);
	m_ListClear = CreateButton(frame, L"Clear");
	m_ListClear->AddClickEvent(this, &MainWindow::ListClearClick);

	return root;
}

MQFrame *MainWindow::CreateLabelFrame(MQWidgetBase *parent)
{
	MQFrame *root = CreateVerticalFrame(parent);

	MQLabel *label;
	label = CreateLabel(root, L"Left");
	label->SetAlignment(MQLabel::ALIGN_LEFT);

	label = CreateLabel(root, L"Center");
	label->SetAlignment(MQLabel::ALIGN_CENTER);

	label = CreateLabel(root, L"Right");
	label->SetAlignment(MQLabel::ALIGN_RIGHT);

	label = CreateLabel(root, L"Frame");
	label->SetFrame(true);

	label = CreateLabel(root, L"Vertical");
	label->SetVerticalText(true);

	label = CreateLabel(root);
	label->SetText(L"WordWrap WordWrap WordWrap WordWrap WordWrap WordWrap WordWrap WordWrap WordWrap WordWrap");
	label->SetWordWrap(true);
	label->SetHintSizeRateX(12);
	label->SetHintSizeRateY(4);

	return root;
}

MQFrame *MainWindow::CreateEditFrame(MQWidgetBase *parent)
{
	MQFrame *root = CreateVerticalFrame(parent);

	MQEdit *edit;
	MQFrame *frame;
	edit = CreateEdit(root, L"Normal");
	edit->AddChangedEvent(this, &MainWindow::EditChanged);
	edit->AddChangingEvent(this, &MainWindow::EditChanging);

	frame = CreateHorizontalFrame(root);
	m_ReadOnlyCheck = CreateCheckBox(frame, L"ReadOnly");
	m_ReadOnlyCheck->SetChecked(true);
	m_ReadOnlyCheck->AddChangedEvent(this, &MainWindow::ReadOnlyCheckChanged);
	m_ReadOnlyEdit = CreateEdit(frame);
	m_ReadOnlyEdit->SetText(L"ReadOnly");
	m_ReadOnlyEdit->SetReadOnly(true);
	m_ReadOnlyEdit->SetHorzLayout(MQWidgetBase::LAYOUT_FILL);
	m_ReadOnlyEdit->AddChangedEvent(this, &MainWindow::EditChanged);
	m_ReadOnlyEdit->AddChangingEvent(this, &MainWindow::EditChanging);

	frame = CreateHorizontalFrame(root);
	m_PasswordCheck = CreateCheckBox(frame, L"Password");
	m_PasswordCheck->SetChecked(true);
	m_PasswordCheck->AddChangedEvent(this, &MainWindow::PasswordCheckChanged);
	m_PasswordEdit = CreateEdit(frame);
	m_PasswordEdit->SetText(L"Password");
	m_PasswordEdit->SetPassword(true);
	m_PasswordEdit->SetHorzLayout(MQWidgetBase::LAYOUT_FILL);
	m_PasswordEdit->AddChangedEvent(this, &MainWindow::EditChanged);
	m_PasswordEdit->AddChangingEvent(this, &MainWindow::EditChanging);

	frame = CreateHorizontalFrame(root);
	CreateLabel(frame, L"Int");
	edit = CreateEdit(frame);
	edit->SetText(L"0");
	edit->SetAlignment(MQEdit::ALIGN_LEFT);
	edit->SetNumeric(MQEdit::NUMERIC_INT);
	edit->AddChangedEvent(this, &MainWindow::EditChanged);
	edit->AddChangingEvent(this, &MainWindow::EditChanging);

	CreateLabel(frame, L"Double");
	edit = CreateEdit(frame);
	edit->SetText(L"0.0");
	edit->SetNumeric(MQEdit::NUMERIC_DOUBLE);
	edit->SetAlignment(MQEdit::ALIGN_RIGHT);
	edit->AddChangedEvent(this, &MainWindow::EditChanged);
	edit->AddChangingEvent(this, &MainWindow::EditChanging);

	frame = CreateHorizontalFrame(root);
	edit = CreateEdit(frame, L"Column 8");
	edit->SetVisibleColumn(8);
	edit = CreateEdit(frame, L"Column 16");
	edit->SetVisibleColumn(16);

	frame = CreateHorizontalFrame(root);
	m_Memo = CreateMemo(frame);
	m_Memo->SetHintSizeRateY(6);
	m_Memo->SetHorzLayout(MQWidgetBase::LAYOUT_FILL);
	m_Memo->AddChangedEvent(this, &MainWindow::MemoChanged);
	m_Memo->AddChangingEvent(this, &MainWindow::MemoChanging);
	frame = CreateVerticalFrame(frame);
	m_MemoAddButton = CreateButton(frame, L"Add");
	m_MemoAddButton->AddClickEvent(this, &MainWindow::MemoAddClick);
	m_MemoClearButton = CreateButton(frame, L"Clear");
	m_MemoClearButton->AddClickEvent(this, &MainWindow::MemoClearClick);

	return root;
}

MQFrame *MainWindow::CreateSpinBoxFrame(MQWidgetBase *parent)
{
	MQFrame *root = CreateVerticalFrame(parent);

	MQSpinBox *spin;
	spin = CreateSpinBox(root);
	spin->SetMin(10);
	spin->SetMax(100);
	spin->SetPosition(50);
	spin->SetIncrement(5);
	spin->AddChangedEvent(this, &MainWindow::SpinBoxChanged);
	spin->AddChangingEvent(this, &MainWindow::SpinBoxChanging);

	MQFrame *frame = CreateHorizontalFrame(root);
	CreateLabel(frame, L"VisibleColumn");
	m_VisibleColumnSpin = CreateSpinBox(frame);
	m_VisibleColumnSpin->SetPosition(8);
	m_VisibleColumnSpin->SetVisibleColumn(8);
	m_VisibleColumnSpin->AddChangedEvent(this, &MainWindow::VisibleColumnSpinChanged);
	m_VisibleColumnSpin->AddChangingEvent(this, &MainWindow::VisibleColumnSpinChanged);

	int r,g,b,a;
	MQWidgetBase::GetDefaultFrameColor(r,g,b,a);
	frame = CreateHorizontalFrame(root);
	frame->SetBackColor(r,g,b,a);
	frame->SetHintSizeRateY(0.05);

	frame = CreateHorizontalFrame(root);
	CreateLabel(frame, L"DecimalDigit");
	CreateLabel(frame, L"Digit");
	m_DigitSpin = CreateSpinBox(frame);
	m_DigitSpin->SetMin(0);
	m_DigitSpin->SetMax(20);
	m_DigitSpin->SetPosition(3);
	m_DigitSpin->AddChangedEvent(this, &MainWindow::DigitSpinChanged);
	m_DigitSpin->AddChangingEvent(this, &MainWindow::DigitSpinChanged);
	CreateLabel(frame, L"Value");
	m_DigitValueSpin = CreateDoubleSpinBox(frame);
	m_DigitValueSpin->SetMin(0.5);
	m_DigitValueSpin->SetMax(50);
	m_DigitValueSpin->SetPosition(0.12345678);
	m_DigitValueSpin->SetDecimalDigit(3);
	m_DigitValueSpin->SetIncrement(0.5);
	m_DigitValueSpin->AddChangedEvent(this, &MainWindow::DigitValueChanged);

	MQDoubleSpinBox *dspin;
	frame = CreateHorizontalFrame(root);
	CreateLabel(frame, L"Exponential");
	dspin = CreateDoubleSpinBox(frame);
	dspin->SetExponential(true);
	dspin->SetMin(0.0001);
	dspin->SetMax(10000);
	dspin->SetPosition(1);
	dspin->SetIncrement(1.1);
	dspin->AddChangedEvent(this, &MainWindow::DSpinBoxChanged);
	dspin->AddChangingEvent(this, &MainWindow::DSpinBoxChanging);

	frame = CreateHorizontalFrame(root);
	dspin = CreateDoubleSpinBox(frame);
	dspin->SetVisibleColumn(8);
	dspin = CreateDoubleSpinBox(frame);
	dspin->SetVisibleColumn(16);

	return root;
}

MQFrame *MainWindow::CreateSliderFrame(MQWidgetBase *parent)
{
	MQFrame *root = CreateVerticalFrame(parent);

	m_Slider = CreateSlider(root);
	m_Slider->SetMin(10);
	m_Slider->SetMax(100);
	m_Slider->SetPosition(50);
	m_Slider->AddChangedEvent(this, &MainWindow::SliderChanged);
	m_Slider->AddChangingEvent(this, &MainWindow::SliderChanging);

	MQFrame *frame;
	MQLabel *label;
	int r,g,b,a;
	MQWidgetBase::GetDefaultFrameColor(r,g,b,a);
	frame = CreateHorizontalFrame(root);
	frame->SetBackColor(r,g,b,a);
	frame->SetHintSizeRateY(0.05);

	m_ScrollBar = CreateScrollBar(root);
	m_ScrollBar->SetMin(10);
	m_ScrollBar->SetMax(100);
	m_ScrollBar->SetPosition(50);
	m_ScrollBar->AddChangedEvent(this, &MainWindow::ScrollBarChanged);
	m_ScrollBar->AddChangingEvent(this, &MainWindow::ScrollBarChanging);

	frame = CreateHorizontalFrame(root);
	CreateLabel(frame, L"Min");
	m_ScrollMinSpin = CreateSpinBox(frame);
	m_ScrollMinSpin->SetMin(-1000);
	m_ScrollMinSpin->SetMax(1000);
	m_ScrollMinSpin->SetPosition(m_ScrollBar->GetMin());
	m_ScrollMinSpin->AddChangedEvent(this, &MainWindow::ScrollSpinChanged);
	m_ScrollMinSpin->AddChangingEvent(this, &MainWindow::ScrollSpinChanged);
	label = CreateLabel(frame, L"Max");
	label->SetFillBeforeRate(1.0);
	m_ScrollMaxSpin = CreateSpinBox(frame);
	m_ScrollMaxSpin->SetMin(-1000);
	m_ScrollMaxSpin->SetMax(1000);
	m_ScrollMaxSpin->SetPosition(m_ScrollBar->GetMax());
	m_ScrollMaxSpin->AddChangedEvent(this, &MainWindow::ScrollSpinChanged);
	m_ScrollMaxSpin->AddChangingEvent(this, &MainWindow::ScrollSpinChanged);
	frame = CreateHorizontalFrame(root);
	CreateLabel(frame, L"Page");
	m_ScrollPageSpin = CreateSpinBox(frame);
	m_ScrollPageSpin->SetMin(1);
	m_ScrollPageSpin->SetMax(1000);
	m_ScrollPageSpin->SetPosition(m_ScrollBar->GetPage());
	m_ScrollPageSpin->AddChangedEvent(this, &MainWindow::ScrollSpinChanged);
	m_ScrollPageSpin->AddChangingEvent(this, &MainWindow::ScrollSpinChanged);
	CreateLabel(frame, L"Increment");
	m_ScrollIncSpin = CreateSpinBox(frame);
	m_ScrollIncSpin->SetMin(1);
	m_ScrollIncSpin->SetMax(1000);
	m_ScrollIncSpin->SetPosition(m_ScrollBar->GetIncrement());
	m_ScrollIncSpin->AddChangedEvent(this, &MainWindow::ScrollSpinChanged);
	m_ScrollIncSpin->AddChangingEvent(this, &MainWindow::ScrollSpinChanged);

	return root;
}

MQFrame *MainWindow::CreateColorFrame(MQWidgetBase *parent)
{
	MQFrame *root = CreateVerticalFrame(parent);

	m_ColorPanel = CreateColorPanel(root);
	m_ColorPanel->SetColor(0, 128, 255);
	m_ColorPanel->AddChangedEvent(this, &MainWindow::ColorPanelChanged);
	m_ColorPanel->AddChangingEvent(this, &MainWindow::ColorPanelChanging);

	return root;
}


BOOL MainWindow::TabChanged(MQWidgetBase *sender, MQDocument doc)
{
	wchar_t text[64];
	swprintf_s(text, L"Tab changed %d", m_Tab->GetCurrentPage());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::ButtonClick(MQWidgetBase *sender, MQDocument doc)
{
	for(int i=0; i<9; i++){
		m_Buttons[i]->SetDown(m_Buttons[i] == sender);
	}
	wchar_t text[64];
	swprintf_s(text, L"%s clicked", static_cast<MQButton*>(sender)->GetText().c_str());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::ToggleButtonClick(MQWidgetBase *sender, MQDocument doc)
{
	static int count = 0;
	wchar_t text[64];
	swprintf_s(text, L"Toggle button clicked %s %d", m_ToggleButton->GetDown() ? L"down" : L"up", ++count);
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::RepeatButtonClick(MQWidgetBase *sender, MQDocument doc)
{
	m_BottomLabel->SetText(L"Repeat button clicked");
	return FALSE;
}

BOOL MainWindow::RepeatButtonRepeat(MQWidgetBase *sender, MQDocument doc)
{
	static int count = 0;
	wchar_t text[64];
	swprintf_s(text, L"Repeat button repeat %d", ++count);
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::OpenFileDialog(MQWidgetBase *sender, MQDocument doc)
{
	MQOpenFileDialog dlg(*this);
	dlg.AddFilter(L"Text file (*.txt)|*.txt");
	dlg.AddFilter(L"Bitmap file (*.bmp)|*.bmp");
	dlg.AddFilter(L"All files (*.*)|*.*");
	if(dlg.Execute()){
		std::wstring fn = L"File name: " + dlg.GetFileName();
		m_BottomLabel->SetText(fn);
	}
	return FALSE;
}

BOOL MainWindow::CheckChanged(MQWidgetBase *sender, MQDocument doc)
{
	wchar_t text[64];
	swprintf_s(text, L"Check changed %s", m_Check->GetChecked() ? L"1" : L"0");
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::RadioChanged(MQWidgetBase *sender, MQDocument doc)
{
	for(int i=0; i<3; i++){
		m_Radio[i]->SetChecked(m_Radio[i] == sender);
	}
	return FALSE;
}

BOOL MainWindow::ComboChanged(MQWidgetBase *sender, MQDocument doc)
{
	wchar_t text[64];
	swprintf_s(text, L"Combo changed %d", m_Combo->GetCurrentIndex());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::ComboAddClick(MQWidgetBase *sender, MQDocument doc)
{
	static int count = 0;
	wchar_t name[64];
	swprintf_s(name, L"Add %d", ++count);
	int index = m_Combo->AddItem(name);
	m_Combo->SetCurrentIndex(index);
	return FALSE;
}

BOOL MainWindow::ComboDeleteClick(MQWidgetBase *sender, MQDocument doc)
{
	int index = m_Combo->GetCurrentIndex();
	if(index >= 0 && index < m_Combo->GetItemCount()){
		m_Combo->DeleteItem(index);
	}
	return FALSE;
}

BOOL MainWindow::ComboClearClick(MQWidgetBase *sender, MQDocument doc)
{
	m_Combo->ClearItems();
	return FALSE;
}

BOOL MainWindow::ListChanged(MQWidgetBase *sender, MQDocument doc)
{
	wchar_t text[64];
	swprintf_s(text, L"List changed %d", m_List->GetCurrentIndex());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::ListAddClick(MQWidgetBase *sender, MQDocument doc)
{
	static int count = 0;
	wchar_t name[64];
	swprintf_s(name, L"Add %d", ++count);
	int index = m_List->AddItem(name);
	m_List->SetCurrentIndex(index);
	return FALSE;
}

BOOL MainWindow::ListDeleteClick(MQWidgetBase *sender, MQDocument doc)
{
	int index = m_List->GetCurrentIndex();
	if(index >= 0 && index < m_List->GetItemCount()){
		m_List->DeleteItem(index);
	}
	return FALSE;
}

BOOL MainWindow::ListClearClick(MQWidgetBase *sender, MQDocument doc)
{
	m_List->ClearItems();
	return FALSE;
}

BOOL MainWindow::EditChanged(MQWidgetBase *sender, MQDocument doc)
{
	MQEdit *edit = static_cast<MQEdit*>(sender);
	std::wstring text = edit->GetText();
	text = L"Edit changed " + text;
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::EditChanging(MQWidgetBase *sender, MQDocument doc)
{
	MQEdit *edit = static_cast<MQEdit*>(sender);
	std::wstring text = edit->GetText();
	text = L"Edit changing " + text;
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::ReadOnlyCheckChanged(MQWidgetBase *sender, MQDocument doc)
{
	m_ReadOnlyEdit->SetReadOnly(m_ReadOnlyCheck->GetChecked());

	wchar_t text[64];
	swprintf_s(text, L"Edit read only %d", m_ReadOnlyEdit->GetReadOnly() ? 1 : 0);
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::PasswordCheckChanged(MQWidgetBase *sender, MQDocument doc)
{
	m_PasswordEdit->SetPassword(m_PasswordCheck->GetChecked());

	wchar_t text[64];
	swprintf_s(text, L"Edit password %d", m_PasswordEdit->GetPassword() ? 1 : 0);
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::MemoChanged(MQWidgetBase *sender, MQDocument doc)
{
	MQMemo *memo = static_cast<MQMemo*>(sender);
	std::wstring text = memo->GetText();
	text = L"Memo changed " + text;
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::MemoChanging(MQWidgetBase *sender, MQDocument doc)
{
	MQMemo *memo = static_cast<MQMemo*>(sender);
	std::wstring text = memo->GetText();
	text = L"Memo changing " + text;
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::MemoAddClick(MQWidgetBase *sender, MQDocument doc)
{
	m_Memo->AddLine(L"AddLine");
	return FALSE;
}

BOOL MainWindow::MemoClearClick(MQWidgetBase *sender, MQDocument doc)
{
	m_Memo->SetText(L"");
	return FALSE;
}

BOOL MainWindow::SpinBoxChanged(MQWidgetBase *sender, MQDocument doc)
{
	MQSpinBox *spin = static_cast<MQSpinBox*>(sender);
	wchar_t text[64];
	swprintf_s(text, L"SpinBox changed %d", spin->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::SpinBoxChanging(MQWidgetBase *sender, MQDocument doc)
{
	MQSpinBox *spin = static_cast<MQSpinBox*>(sender);
	wchar_t text[64];
	swprintf_s(text, L"SpinBox changeing %d", spin->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::VisibleColumnSpinChanged(MQWidgetBase *sender, MQDocument doc)
{
	m_VisibleColumnSpin->SetVisibleColumn(m_VisibleColumnSpin->GetPosition());

	wchar_t text[64];
	swprintf_s(text, L"SpinBox visible column %d", m_VisibleColumnSpin->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::DigitSpinChanged(MQWidgetBase *sender, MQDocument doc)
{
	m_DigitValueSpin->SetDecimalDigit(m_DigitSpin->GetPosition());

	wchar_t text[64];
	swprintf_s(text, L"DoubleSpinBox decimal digit %d", m_DigitSpin->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::DigitValueChanged(MQWidgetBase *sender, MQDocument doc)
{
	m_DigitSpin->SetPosition(m_DigitValueSpin->GetDecimalDigit());

	wchar_t text[64];
	swprintf_s(text, L"DoubleSpinBox decimal digit %d", m_DigitValueSpin->GetDecimalDigit());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::DSpinBoxChanged(MQWidgetBase *sender, MQDocument doc)
{
	MQDoubleSpinBox *spin = static_cast<MQDoubleSpinBox*>(sender);
	wchar_t text[64];
	swprintf_s(text, L"DoubleSpinBox changed %f", spin->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::DSpinBoxChanging(MQWidgetBase *sender, MQDocument doc)
{
	MQDoubleSpinBox *spin = static_cast<MQDoubleSpinBox*>(sender);
	wchar_t text[64];
	swprintf_s(text, L"DoubleSpinBox changing %f", spin->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::SliderChanged(MQWidgetBase *sender, MQDocument doc)
{
	wchar_t text[64];
	swprintf_s(text, L"Slider changed %f", m_Slider->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::SliderChanging(MQWidgetBase *sender, MQDocument doc)
{
	wchar_t text[64];
	swprintf_s(text, L"Slider changing %f", m_Slider->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::ScrollBarChanged(MQWidgetBase *sender, MQDocument doc)
{
	wchar_t text[64];
	swprintf_s(text, L"ScrollBar changed %d", m_ScrollBar->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::ScrollBarChanging(MQWidgetBase *sender, MQDocument doc)
{
	wchar_t text[64];
	swprintf_s(text, L"ScrollBar changing %d", m_ScrollBar->GetPosition());
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::ScrollSpinChanged(MQWidgetBase *sender, MQDocument doc)
{
	m_ScrollBar->SetMin(m_ScrollMinSpin->GetPosition());
	m_ScrollBar->SetMax(m_ScrollMaxSpin->GetPosition());
	m_ScrollBar->SetPage(m_ScrollPageSpin->GetPosition());
	m_ScrollBar->SetIncrement(m_ScrollIncSpin->GetPosition());
	return FALSE;
}

BOOL MainWindow::ColorPanelChanged(MQWidgetBase *sender, MQDocument doc)
{
	int r,g,b;
	m_ColorPanel->GetColor(r,g,b);
	double h,s,v;
	m_ColorPanel->GetHSV(h,s,v);

	wchar_t text[64];
	swprintf_s(text, L"ColorPanel changed RGB(%d %d %d) HSV(%.0lf %.2lf %.2lf)", r,g,b, h,s,v);
	m_BottomLabel->SetText(text);
	return FALSE;
}

BOOL MainWindow::ColorPanelChanging(MQWidgetBase *sender, MQDocument doc)
{
	int r,g,b;
	m_ColorPanel->GetColor(r,g,b);
	double h,s,v;
	m_ColorPanel->GetHSV(h,s,v);

	wchar_t text[64];
	swprintf_s(text, L"ColorPanel changing RGB(%d %d %d) HSV(%.0lf %.2lf %.2lf)", r,g,b, h,s,v);
	m_BottomLabel->SetText(text);
	return FALSE;
}



